#!/bin/bash
# $Id: install.sh,v 1.1 2000/12/17 14:21:38 amos Exp $
# Script to install Apollo

PREFIX="/usr"
VERSION=1.0

for arg in $*; do
    case $arg in
	--help|-h)
	    echo "Usage: $0 [options]"
	    echo
	    echo "Options: -h"
	    echo "         --help               This message"
	    echo "         --with-kde[=kde-dir] Install KDE icons and menu item,"
	    echo "                              default is $KDEDIR"
	    echo "         --prefix=[dir]       Use dir as prefix for installation,"
	    echo "                              default is /usr"
	    echo "         --uninstall          Uninstall an existing apollo installation"
	    exit 1
	    ;;
	--with-kde*)
	    if echo $arg | grep -e "--with-kde=" >/dev/null; then
		KDEDIR=`echo $arg | sed 's/--with-kde=/\1/'`
		KDE_INSTALL="true";
	    else
		KDE_INSTALL="true";
	    fi
	    ;;
	--prefix=*)
	    PREFIX=`echo $arg | sed 's/--prefix=/\1/'`
	    ;;
	--uninstall)
	    UNINSTALL="true"
	    ;;
	*)
  	    echo "$arg: unkown option"
              $0 -h
  	    exit 1
	    ;;
    esac;
done

if [ -z "$KDEDIR" ] ; then
	KDEDIR="/usr/"
fi

BINDIR="$PREFIX/bin"
DOCDIR="$PREFIX/doc/apollo-$VERSION"
DOCS="CREDITS GPL INSTALL README"

if [ ! -z "$UNINSTALL" ]; then
    echo "Uninstalling binaries from $BINDIR"

    rm -f $BINDIR/apollo

    echo "Uninstalling documentation from $DOCDIR"

    for i in $DOCS; do
	rm -f $DOCDIR/$i
    done

    rmdir $DOCDIR

    if [ -z "$KDE_INSTALL" ]; then
	echo "Skipping KDE uninstall"
    else
	echo "Uninstalling KDE icons and menu items from $KDEDIR"

	for i in 16 22 32 48; do
	    rm -f $KDEDIR/share/icons/hicolor/$i"x"$i/apps/apollo.png
	done
	for i in 16 22 32; do
	    rm -f $KDEDIR/share/icons/locolor/$i"x"$i/apps/apollo.png
	done

	rm -f $KDEDIR/share/applnk/Multimedia/apollo.desktop
    fi

    echo "Apollo is now uninstalled"

    exit
fi

echo "Installing binaries to $BINDIR"

install -m 755 apollo $BINDIR || exit 1

echo "Installing documentation to $DOCDIR"
echo

mkdir -p $DOCDIR || exit 1
for i in $DOCS; do
    cp -f $i $DOCDIR || exit 1
done

if [ -z "$KDE_INSTALL" ]; then
    echo "Skipping KDE install"
else
    echo "Installing KDE icons and menu items to $KDEDIR"

    for i in 16 22 32 48; do
	mkdir -p $KDEDIR/share/icons/hicolor/$i"x"$i/apps || exit 1
    done
    for i in 16 22 32; do
	mkdir -p $KDEDIR/share/icons/locolor/$i"x"$i/apps || exit 1
    done

    for i in 16 22 32 48; do
	cp -f icons/hicolor/$i"x"$i/apps/apollo.png $KDEDIR/share/icons/hicolor/$i"x"$i/apps || exit 1
    done
    for i in 16 22 32; do
	cp -f icons/locolor/$i"x"$i/apps/apollo.png $KDEDIR/share/icons/locolor/$i"x"$i/apps || exit 1
    done

    cat > $KDEDIR/share/applnk/Multimedia/apollo.desktop <<EOF
[Desktop Entry]
Comment=Apollo music player
Exec=apollo
Icon=apollo
Name=Apollo
Path=
Terminal=0
TerminalOptions=
Type=Application
X-KDE-SubstituteUID=false
X-KDE-Username=
EOF

fi

echo
echo "If $BINDIR is not in your executable path, add"
echo "export PATH="'$'"PATH:$BINDIR"
echo "to ~/.bash_profile"
echo

echo "Done..."
echo
echo "Type apollo to start Apollo version $VERSION"
echo "and be sure to visit http://www.apolloplayer.org for information and updates"
