/* ;-*-c++-*-
 * qmconfig.h
 *
 * $Id: qmconfig.h,v 1.7 2000/12/14 16:14:46 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef QMCONFIG_H_INCLUDED
#define QMCONFIG_H_INCLUDED

#include <qrect.h>
#include <qstring.h>
#include <memory>

/*! \file qmconfig.h
  Definition of QmConfig class
*/

class QTextStream;
class QSize;
class QPoint;
class QDomNode;

class QmConfig
{
public:
    ~QmConfig();

    static QmConfig *instance();

    void load();
    void save();

    void setWindowPosition( const QPoint &pos );
    void setWindowSize( const QSize &size );
    void setFileDialogPosition( const QPoint &pos );
    void setFileDialogSize( const QSize &size );
    void setScrollPosition( const QPoint &pos );
    void setCurrentMusicPath( const QString &path );
	void setPlayingSong( const QString &song );
    void setMpg123Path( const QString &path );

    QPoint windowPosition() const;
    QSize windowSize() const;
    QPoint fileDialogPosition() const;
    QSize fileDialogSize() const;
    const QPoint &scrollPosition() const;
    const QString &currentMusicPath() const;
	const QString &playingSong() const;
    const QString &mpg123Path() const;

private:
    QString configPath() const;
    QString configFileName() const;

private:
    QmConfig();

    void writeWindow( QTextStream &conf_stream, int level );
    void writeFileDialog( QTextStream &conf_stream, int level );
    void writePath( const QString &name, const QString &path, QTextStream &conf_stream, int level );
	void writePlayingSong( QTextStream &conf_stream, int level );

    void writeWindowGeometry( const QRect &rect, QTextStream &conf_stream, int level );
    void writeWindowPosition( const QPoint &pos, QTextStream &conf_stream, int level );
    void writeWindowSize( const QSize &size, QTextStream &conf_stream, int level );
    void writePathConfig( QTextStream &conf_stream, int level );

    void readWindow( const QDomNode &node );
    void readFileDialog( const QDomNode &node );
    void readPathConfig( const QDomNode &node );
	QString readPlayingSong ( const QDomNode &node ) const;

    QRect readGeometry( const QDomNode &node ) const;
    QPoint readPosition( const QDomNode &node ) const;
    QString readPath( const QDomNode &node ) const;

    int nodeAsInteger( const QDomNode &node ) const;

    QString indent( int level ) const;

private:
    QRect 		m_WindowGeometry;
    QRect 		m_FileDialogGeometry;
    QPoint      m_ScrollPos;
    QString 	m_CurrentMusicPath;
	QString 	m_PlayingSong;
    QString     m_Mpg123Path;

    static auto_ptr<QmConfig> s_pInstance;
};


#endif // QMCONFIG_H_INCLUDED
