/* ;-*-c++-*-
 * qmplayer.h
 *
 * $Id: qmplayer.h,v 1.6 2000/12/09 12:12:36 amos Exp $
 *
 * Apollo sound player: http://www.apolloplayer.org
 * Copyright(C) 2000 Apollo Team.  See CREDITS file.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * The GNU General Public License is also available online at:
 *
 * http://www.gnu.org/copyleft/gpl.html
 */

#ifndef QMPLAYER_H_INCLUDED
#define QMPLAYER_H_INCLUDED

#include <qobject.h>

class QString;

/*! \file qmplayer.h
  Definition of QmPlayer class which provides the interface for all players.
*/

/*!
  \class QmPlayer qmplayer.h
  \brief The player interface.

  This class is the base class of all players.  It provides functionality to
  control the playing of a song.  Note that it does not control traversing
  songs, as that is left to the QmPlayList.

  The QmMainWindow controls the player through the interface provided by
  this class.

  \todo Implement a QmPlayerFactory to create various player instances
  depending on the song format.
  \todo Implement a QmSongInfo abstract class.  Derive from it according to
  the format of the song.  Information will be extracted from the QmSongInfo
  class to be used in QmMainWindow.
*/

class QmPlayer : public QObject
{
	Q_OBJECT
public:
    QmPlayer(QObject *parent = 0, const char *name = 0) : QObject(parent, name) {};
    virtual ~QmPlayer() {};

	/*!
	  Play the song \a Filename.
	*/
	
    virtual void play(const QString &Filename) = 0;

	/*!
	  Stop the playing.
	*/
	
    virtual void stop() = 0;

	/*!
	  Pause the playing.
	*/
	
    virtual void pause() = 0;

signals:
	void playingStopped();
};

#endif // QMPLAYER_H_INCLUDED
