/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include <string.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <gmodule.h>
#include "mixmagic.h"
#include "plugins.h"

static void plugins_scan_dir (gchar *dirname);

static GList *output_plugins = NULL;

/*
 * Initializes the plugin system
 */
void
plugins_init (void)
{
	plugins_scan_dir (MIXMAGIC_PLUGIN_DIR);
}

void
plugins_register_output_plugin (OutputPlugin *output)
{
	if (output->version != OUTPUT_PLUGIN_VERSION) {

		g_print ("Can't load plug-in '%s'.\nPlugin version = %d, version required = %d\n", 
			 g_module_name (output->module),
			 output->version, 
			 OUTPUT_PLUGIN_VERSION);
		return;
	}
	output_plugins = g_list_prepend (output_plugins, output);

	g_print ("Output plug-in registered: %s\n", output->info ? output->info : "No info");
}

void
plugins_unregister_output_plugin (OutputPlugin *output)
{
	g_print ("Output plug-in registered: %s\n", output->info ? output->info : "No info");

	output_plugins = g_list_remove (output_plugins, output);
}

static void
plugins_scan_dir (gchar *dirname)
{
        gchar *filename, *ext;
        DIR *dir;       
        struct dirent *ent;
        struct stat statbuf;
	
        dir = opendir(dirname);
        if (dir) {
		while ((ent = readdir(dir)) != NULL) {
			filename = g_strdup_printf ("%s/%s", dirname, ent->d_name);
			if (!stat(filename, &statbuf)) {
				if (S_ISREG(statbuf.st_mode)) {
					if ((ext = strrchr(ent->d_name, '.')) != NULL) {
						if (!strcmp(ext, ".so")) {
							if (g_module_open (filename, 0) == NULL) {
								g_print ("Loading of plug-in '%s' failed with the following error '%s'\n", filename, g_module_error ());
							}
						}
					}
				}
			}
			g_free (filename);
		}
	}
}

/*
 * Free the plugins!
*/
void
plugins_free (void)
{
}

GList *
plugins_get_output_list ()
{
	return output_plugins;
}
