/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include "mixmagic.h"
#include "plugins.h"

#include <gnome.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>

#ifdef HAVE_SYS_SOUNDCARD_H
#include <sys/soundcard.h>
#elif defined(HAVE_MACHINE_SOUNDCARD_H)
#include <machine/soundcard.h>
#endif

static void oss_close (void);

static gint fd = -1;
static OutputPlugin *plugin = NULL;

static void
oss_about (void)
{
	GtkWidget *about, *uri;
	const gchar *authors[] = {"Jonas Borgstrm", NULL};

	about = gnome_about_new ("MixMagic OSS output plug-in.", VERSION,
                                 "(C) 2000 Jonas Borgstrm",
                                 authors, "This output plug-in writes the audio output to /dev/dsp", NULL);

        uri = gnome_href_new ("http://mixmagic.sourceforge.net/",
                            _("MixMagic Home Page"));

        gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
                            uri, TRUE, FALSE, 0);
        gtk_widget_show (uri);

        gnome_dialog_set_close (GNOME_DIALOG (about), TRUE);
	gtk_widget_show (about);
}

static gboolean
oss_open (AudioFormat format, gint rate, gint num_channels)
{
	gint bits = 16;

	/* Attempt to open the audio device */

	if (fd == -1)
		oss_close ();

	fd = open("/dev/dsp", O_WRONLY);

	if(fd == -1) {

		perror("/dev/dsp");
		return FALSE;
	}

        num_channels -= 1;

	switch (format) {
	case AFORMAT_U8:
	case AFORMAT_S8:
		bits = 8;
		break;
	case AFORMAT_U16_LE:
	case AFORMAT_S16_LE:
	case AFORMAT_U16_BE:
	case AFORMAT_S16_BE:
		bits = 16;
		break;
	};
	if (ioctl (fd, SNDCTL_DSP_SAMPLESIZE, &bits) < 0) {

		perror("Error setting sample size!");
		return FALSE;
	}
	if (ioctl (fd, SNDCTL_DSP_STEREO, &num_channels) < 0) {

		perror("Error setting stereo mode!");
		return FALSE;
	}
	if(ioctl(fd, SNDCTL_DSP_SPEED, &rate) < 0) {

		perror("Error setting rate!");
		return FALSE;
	}   
	return TRUE;
}

static void
oss_write (gpointer data, gint size)
{
	write (fd, data, size);
}

static void
oss_close (void)
{
	if (fd != -1)
		close (fd);
}


const gchar* 
g_module_unload ()
{
	plugins_unregister_output_plugin (plugin);
}

const gchar* 
g_module_check_init (GModule *module)
{
	plugin = g_new0 (OutputPlugin, 1);

	plugin->module = module;
	plugin->info = "MixMagic OSS output plug-in.";
	plugin->version = OUTPUT_PLUGIN_VERSION;

	plugin->about = oss_about;
	plugin->configure = NULL;

	plugin->open = oss_open;
	plugin->write = oss_write;
	plugin->close = oss_close;

	plugins_register_output_plugin (plugin);

	return NULL;
}
