/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include <gnome.h>
#include <stdio.h>
#include "mixmagic.h"
#include "plugins.h"
#include "sample.h"

static void wavwriter_close (void);

FILE *fd = NULL;
static OutputPlugin *plugin = NULL;
gint written;
Wavheader header;

static void
wavwriter_about (void)
{
	GtkWidget *about, *uri;
	const gchar *authors[] = {"Jonas Borgstrm", NULL};

	about = gnome_about_new ("MixMagic Wavwriter output plug-in.", VERSION,
                                 "(C) 2000 Jonas Borgstrm",
                                 authors, "This output plug-in writes the audio output to the file $HOME/mixmagic.wav", NULL);

        uri = gnome_href_new ("http://mixmagic.sourceforge.net/",
                            _("MixMagic Home Page"));

        gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (about)->vbox),
                            uri, TRUE, FALSE, 0);
        gtk_widget_show (uri);

        gnome_dialog_set_close (GNOME_DIALOG (about), TRUE);
	gtk_widget_show (about);
}

static gboolean
wavwriter_open (AudioFormat format, gint rate, gint num_channels)
{
	gint bits = 16;
	gchar *filename;

	/* Attempt to open the audio device */

	if (fd != NULL)
		wavwriter_close ();

	filename = g_strdup_printf ("%s%c%s", g_get_home_dir (), G_DIR_SEPARATOR, "mixmagic.wav");

	if ((fd = fopen(filename, "wb")) == NULL) {

		g_print ("wavwriter: Can't create %s\n", filename);
		g_free (filename);
		return FALSE;
	}
	g_free (filename);
	written = 0;

	switch (format) {
	case AFORMAT_U8:
	case AFORMAT_S8:
		bits = 8;
		break;
	case AFORMAT_U16_LE:
	case AFORMAT_S16_LE:
	case AFORMAT_U16_BE:
	case AFORMAT_S16_BE:
		bits = 16;
		break;
	};
	memcpy (&header.main_chunk, "RIFF", 4);
	header.length = GUINT32_TO_LE (0);
        memcpy (&header.chunk_type, "WAVE", 4);
        memcpy (&header.sub_chunk, "fmt ", 4);
        header.sc_len = GUINT32_TO_LE (16);
        header.format = GUINT16_TO_LE (1);
        header.modus = GUINT16_TO_LE (num_channels);
        header.sample_fq = GUINT32_TO_LE (rate);
	header.bit_p_spl = GUINT16_TO_LE (bits);
	header.byte_p_sec = GUINT32_TO_LE (rate * header.modus * (GUINT16_FROM_LE (header.bit_p_spl) / 8));
        header.byte_p_spl = GUINT16_TO_LE ((GUINT16_FROM_LE (header.bit_p_spl) / (8 / num_channels)));
        memcpy (&header.data_chunk, "data", 4);
        header.data_length = GUINT32_TO_LE (0);

        fwrite(&header, sizeof (Wavheader), 1, fd);

	return TRUE;
}

static void
wavwriter_write (gpointer data, gint size)
{
	written += fwrite (data, 1, size, fd);
}

static void
wavwriter_close (void)
{
	if (fd == NULL)
		return;

	header.length = GUINT32_TO_LE(written + sizeof (Wavheader) - 4);
	header.data_length = GUINT32_TO_LE(written);

	fseek (fd, SEEK_SET, 0);
        fwrite(&header, sizeof (Wavheader), 1, fd);

	fclose (fd);
	fd = NULL;
}


const gchar* 
g_module_unload ()
{
	plugins_unregister_output_plugin (plugin);
}

const gchar* 
g_module_check_init (GModule *module)
{
	plugin = g_new0 (OutputPlugin, 1);

	plugin->module = module;
	plugin->info = "MixMagic Wavwriter output plug-in.";
	plugin->version = OUTPUT_PLUGIN_VERSION;

	plugin->about = wavwriter_about;
	plugin->configure = NULL;

	plugin->open = wavwriter_open;
	plugin->write = wavwriter_write;
	plugin->close = wavwriter_close;

	plugins_register_output_plugin (plugin);

	return NULL;
}
