/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>

#include "mixmagic.h"
#include "sample.h"
#include "index.h"

/*
 * Create a new Sample.
 */
Sample *
sample_new ()
{
	Sample *sample;

	sample = g_new0 (Sample, 1);

	sample->ref_count = 1;

	return sample;
}

gboolean
sample_init (Sample *sample, GnomeAppBar *appbar)
{
	Wavheader header;

	g_assert (sample != NULL);

	if ((sample->fd = open (sample->filename, O_RDONLY)) == -1) {

		g_print ("Faild to open sample '%s'\n", sample->filename);

		return FALSE;
	}

	if (read (sample->fd, &header, sizeof (Wavheader)) != sizeof (Wavheader)) {

		g_print ("Faild to read header from sample '%s'\n", sample->filename);

		return FALSE;
	}
	if (memcmp (&header.main_chunk, "RIFF", 4) != 0) {
		close (sample->fd);

		return FALSE;
	}
	/* Get filesize */
	sample->map_size = lseek (sample->fd, 0, SEEK_END);

	sample->rate = header.sample_fq;
	sample->byte_p_spl = header.byte_p_spl;
	sample->num_channels = header.modus;
	sample->length = header.data_length;

	sample->map_addr = mmap (0, sample->map_size, 
			     PROT_READ, MAP_SHARED, 
			     sample->fd, 0);

	g_assert (sample->data != (gpointer)-1);

	sample->data = sample->map_addr + sizeof (Wavheader);

	sample->index = index_new (sample, appbar);

	return TRUE;
}

Sample *
sample_ref (Sample *sample)
{
	g_return_val_if_fail (sample != NULL, NULL);
        g_return_val_if_fail (sample->ref_count > 0, NULL);

        sample->ref_count++;
        return sample;
}

void 
sample_unref (Sample *sample)
{
	sample->ref_count--;

	if (sample->ref_count == 0) {

		g_print ("sample unref\n");

		if (sample->id)
			g_free (sample->id);

		if (sample->filename)
			g_free (sample->filename);

		if (sample->map_addr) {
			if (munmap (sample->map_addr, sample->map_size) == -1)
				g_warning ("munmap (%p, %d) failed\n", 
					   sample->map_addr, 
					   sample->map_size);
		}
		if (sample->fd)
			close (sample->fd);

		index_destroy (sample->index);

		g_free (sample);
	
	}
}
