/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
/*  This file is part of MixMagic.

    Copyright (C) 2000 Jonas Borgstrm <jonas_b@bitsmart.com>.

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/

#ifndef __MIXMAGIC_TRACK_H__
#define __MIXMAGIC_TRACK_H__

#include <gtk/gtk.h>
#include "block.h"


#define TRACK(obj)         GTK_CHECK_CAST(obj, track_get_type (), Track)
#define TRACK_CLASS(klass)  GTK_CHECK_CLASS_CAST(klass, track_get_type (), TrackClass)
#define IS_TRACK(obj)       GTK_CHECK_TYPE(obj, track_get_type ())

#ifndef TRACK_DEFINED
typedef struct _Track Track;
#define TRACK_DEFINED
#endif
typedef struct _TrackClass TrackClass;
typedef enum _PasteMode PasteMode;

enum _PasteMode {
	PASTE_BLOCK,
	PASTE_LOOPBLOCK
};

struct _Track {
        GtkHBox box;

	GtkWidget *layout;
	GtkWidget *popup_menu;
	GtkWidget *marker;
	GtkWidget *insert_file_sel; /* Insert sample dialog */
	GtkWidget *solo_toggle_button;
	GtkWidget *mute_toggle_button;
	GtkWidget *amp_spin_button;
	GtkWidget *pan_spin_button;
	GtkWidget *frame;

	GdkPoint menu_point;

	GList *block_list;

	gint track_num;

	gboolean mute_flag;
	gboolean going_solo_flag; /* This flag is used to avoid infinite loops when "Solo" is toggled */

	PasteMode paste_mode;

	Song *song;

	gfloat amp;
	GtkAdjustment *amp_adj;
	gfloat pan;
	GtkAdjustment *pan_adj;
};

struct _TrackClass 
{
        GtkHBoxClass parent_class;
};

GtkWidget *	track_new		(GtkAdjustment *adjustment, Song *song, gint track_num);
guint		track_get_type		(void);
void		track_add_block		(Track *track, Block *block);
void		track_remove_block	(Track *track, Block *block);
GtkAdjustment*	track_get_adjustment	(Track *track);
void		track_set_adjustment	(Track *track, GtkAdjustment *adjustment);
void		track_update		(Track *track);
void		track_sort_blocks	(Track *track);
void		track_move_marker	(Track *track, gint position);
void		track_show_marker	(Track *track);
void		track_hide_marker	(Track *track);
void 		track_set_length 	(Track *track, gint length);
void		track_set_amp 		(Track *track, gfloat amp);
void		track_set_pan 		(Track *track, gfloat pan);
void		track_set_mute 		(Track *track, gboolean mute);

#endif
